local addon = CreateFrame("Frame","Shako")

local HELM = 1
local CLOAK = 2

local mt = {__index = function() return (ShowingHelm() and HELM or 0) + (ShowingCloak() and CLOAK or 0) end}

function addon:ADDON_LOADED(name)
    if name == "Shako" then
        ShakoDB = setmetatable(ShakoDB or {}, mt)
        self.db = ShakoDB
        
        hooksecurefunc("GearSetButton_OnEnter", function(button) self:GearSetButtonOnEnter(button) end)
		
        self:RegisterEvent("EQUIPMENT_SWAP_FINISHED")
        self:RegisterEvent("PLAYER_LOGOUT")
    end
end

function addon:GearSetButtonOnEnter(button)
	if not button.shakoHelmButton then
		self:SetupUI(button)
	end	
end

local function opButtonOnEnter(button)	
	button:SetAlpha(1)	
	GameTooltip:SetOwner(button, "ANCHOR_RIGHT")
	GameTooltip:SetText(button.tooltip)
end

local function opButtonOnLeave(button)
	button:SetAlpha(0.65)
	GameTooltip:Hide()
end


function addon:OpButtonOnClick(button)
	local set = button.parent.name
	if set and set ~= "" then
		local setting = self.db[set]
		if button:GetChecked() then
			setting = bit.band(setting, bit.bnot(button.type))
		else
			setting = bit.bor(setting, button.type)
		end
		self.db[set] = setting
		
		if button.parent.Check:IsVisible() then -- This set is equipped so apply setting now.
			self:ApplySetting(set)
		end	
	end
end


local function createButton(texturePath, parent, toggleType, tooltip, ...)
	local button = CreateFrame("CheckButton", nil, parent)
	button:SetNormalTexture(texturePath)
	button:SetCheckedTexture("Interface\\Buttons\\UI-GroupLoot-Pass-Up")
	button:GetCheckedTexture():SetBlendMode("BLEND")
	
	button:SetWidth(16)
	button:SetHeight(16)
	button:Hide()
	button:SetAlpha(0.65)	
	
	button:SetScript("OnEnter", opButtonOnEnter)
	button:SetScript("OnLeave", opButtonOnLeave)
	button:SetScript("OnUpdate", opButtonOnUpdate)
	button:SetScript("OnClick", function(button) addon:OpButtonOnClick(button) end)
	
	button.type = toggleType
	button.tooltip = tooltip
	button.parent = parent
	
	button:SetPoint(...)
	
	return button
end

local function showButtons(sibling)
	local button = sibling:GetParent()
	if button.name and button.name ~= "" then -- A button without a set name is the "Add New Set" button
		local setting = addon.db[button.name]
		
		button.shakoHelmButton:Show()
		button.shakoHelmButton:SetChecked(bit.band(setting, HELM) == 0)
		
		button.shakoCloakButton:Show()
		button.shakoCloakButton:SetChecked(bit.band(setting, CLOAK) == 0)	
		
	end

end

local function hideButtons(sibling) 
	local button = sibling:GetParent()
	button.shakoHelmButton:Hide()
	button.shakoCloakButton:Hide()
end


function addon:SetupUI(parent)
	parent.shakoCloakButton = createButton("Interface\\AddOns\\Shako\\inv_misc_cape_11", parent, CLOAK, SHOW_CLOAK, "RIGHT", parent.EditButton, "LEFT", -1, 0)
	parent.shakoHelmButton = createButton("Interface\\AddOns\\Shako\\inv_helmet_03", parent, HELM, SHOW_HELM, "RIGHT", parent.shakoCloakButton, "LEFT", -1, 0)
	
	parent.DeleteButton:HookScript("OnShow", showButtons)
	parent.DeleteButton:HookScript("OnHide", hideButtons)
end

function addon:ApplySetting(set)
	local setting = self.db[set]
	ShowCloak(bit.band(setting, CLOAK)) 
    ShowHelm(bit.band(setting, HELM))
end

function addon:EQUIPMENT_SWAP_FINISHED(completed, name)
    if completed and name then
        self:ApplySetting(name)
    end
end

function addon:PLAYER_LOGOUT() -- Remove any sets that don't exist
    for k,v in pairs(self.db) do
        if not GetEquipmentSetInfoByName(k) then
            self.db[k] = nil
        end
    end
    
end

function addon:OnEvent(event,...)
    if self[event] then return self[event](self,...) end
end

function addon:Init()
    self:SetScript("OnEvent",self.OnEvent)
    self:RegisterEvent("ADDON_LOADED")
end

addon:Init()